// -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vim:ts=2:sw=2:tw=80:et

/* 
 * Copyright (c) 2002, Andrew Carra and Robert Hagemann
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the WiGLE.net nor Mimezine nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.mimezine.geo;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.Toolkit;
import javax.swing.*;
import java.io.*;
import com.sun.image.codec.jpeg.*;

/**
 * The frame that is rendered to. 
 * XXX: should probably actually be abstract.
 */
public class AbstractAppFrame extends Frame {
  public static final String VERSION_ID =
    "$Id: AbstractAppFrame.java,v 1.2 2002/09/19 05:06:09 bobzilla Exp $";

  private static boolean isDrawing;
  private AppController controller;
  private Image offScreenBuffer;
  private Image AABuffer;
  private Thread thread;
  private static final boolean doubleBuffer = true;
  private boolean fromUpdate = false;

  private AbstractAppFrame() {
  }

  protected AbstractAppFrame ( AppController controller ) {
    this( controller, "", "" );
  }

    /**
     * @param controller the AppConroller this is the view for
     * @param title the title of the Frame
     * @param iconName the name of the icon for this frame
     */
  public AbstractAppFrame ( AppController controller, String title, String iconName ) {
    super( title );
    this.controller = controller;

    ImageIcon image = new ImageIcon(iconName);
    this.setIconImage(image.getImage());

    isDrawing = false;
  }

  public void update( Graphics g ) {
    Graphics2D graphics = (Graphics2D) g;
    if ( ! doubleBuffer ) {
      controller.drawRecords( graphics );
      return;
    }

    // Will hold the graphics context from the offScreenBuffer.
    // We need to make sure we keep our offscreen buffer the same size
    // as the graphics context we're working with.
    if ( offScreenBuffer==null ||
        ( ! (offScreenBuffer.getWidth(this) == this.getWidth()
        && offScreenBuffer.getHeight(this) == this.getHeight() ) ) ) {

      offScreenBuffer = this.createImage( getSize().width, getSize().height );
    }

    Graphics2D gr = controller.createGraphics( offScreenBuffer );

    controller.setSize( this.getWidth(), this.getHeight() );
    // Do drawing with gr
    if ( controller.isDirty() ) {
      fromUpdate = true;
      paint( (Graphics) gr );
      fromUpdate = false;
    }

    // And now we transfer the info in the buffer onto the
    // graphics context we got from the browser in one smooth motion.
    graphics.drawImage( offScreenBuffer, 0, 0, this );
  }

  public void paint( Graphics g ) {
    // System.out.println( "paint" );
    if ( ! fromUpdate ) {
      update( g );
      return;
    }

    Graphics2D graphics = (Graphics2D) g;
    if ( controller.isDirty() ) {
      controller.drawRecords( graphics );
    }
  }

  public boolean handleEvent( Event event ) {
    if ( event.id == Event.WINDOW_DESTROY ) {
        controller.shutdown();
    }
    else {
      return controller.handleEvent( event );
    }
    return false;
  }

}
