// -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vim:ts=2:sw=2:tw=80:et

/* 
 * Copyright (c) 2002, Andrew Carra and Robert Hagemann
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the WiGLE.net nor Mimezine nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.mimezine.geo;

import java.awt.Event;
import java.awt.Graphics2D;
import java.awt.Image;

/**
 * control interface for AppFrames.
 */
public interface AppController {
  public static final String VERSION_ID =
    "$Id: AppController.java,v 1.2 2002/09/19 05:06:09 bobzilla Exp $";

  /**
   * plot the Data
   * @param graphics the Graphics2D to draw into
   */
  public void drawRecords( Graphics2D graphics );

  /**
   * sets the frame size in pixels
   *
   * @param sizeX  
   * @param sizeY
   */
  public void setSize( int sizeX, int sizeY );
    
  /**
   * @return is the dataset dirty?
   */
  public boolean isDirty();
    
  /**
   * handle a UI event.
   * @param event the GUI event to handle
   * @return was this event handled?
   */
  public boolean handleEvent( Event event );

  /**
   * create a Graphics2D to work on this Image with.
   *
   * @param image the Image to work on
   * @return the Graphics2D for this Image
   */
  public Graphics2D createGraphics( Image image );
    
  /**
   * UI shutdown event.
   */
  public void shutdown();
}
